'''
% Survival time F-test
% This program conducts survival time F-test, which is used to examine whether two normal populations have same variance or not.
%
% This program requires R, rpy, numpy
%    R ==> http://www.r-project.org/
%    rpy ==> http://rpy.sourceforge.net/
%    numpy ==> http://numpy.scipy.org/
%
% Last Update: 2011.06.03
% By Jae-Seong Yang, destine@postech.ac.kr
'''

import numpy
import rpy
import SurvivalData


def GetFreqData( deathTimes, deathNums ):
    result = []
    for i in range(0, len( deathTimes ) ):
        if i == 0:
            prevTime = 0
        else:
            prevTime = deathTimes[i-1]
        curTime = deathTimes[i]
        if deathNums[i] == 0:
            continue
        bins = numpy.array( range( 1, deathNums[i] + 1 ) )
        step = float( curTime - prevTime ) / deathNums[i]
        result.extend( [ prevTime + x * step for x in bins ] )
    return result

def SurvivalTimeFTest( aData1, aData2 ):
    diff_s_list1 = [ 0 ]
    for i in range( 1, len( aData1.timelist ) ):
        diff_s_list1.append( int( (aData1.km_data_list[i-1].s_hat - aData1.km_data_list[i].s_hat) * aData1.total ) )

    diff_s_list2 = [ 0 ]
    for i in range( 1, len( aData2.timelist ) ):
        diff_s_list2.append( int( (aData2.km_data_list[i-1].s_hat - aData2.km_data_list[i].s_hat) * aData2.total ) )

    deathData1 = GetFreqData( aData1.timelist, diff_s_list1 )
    deathData2 = GetFreqData( aData2.timelist, diff_s_list2 )

    result = rpy.r.var_test( deathData1, deathData2 )
    fvalue = result['statistic']['F']
    variance_ratio = result['estimate']['ratio of variances']
    pvalue = result['p.value']

    return [ variance_ratio, pvalue ]


def RunSurvivalTimeFTestAll( conditions ):
    results_var1Test = []

    for i in range( 0, len( conditions ) ):
        aData1 = conditions[i]
        for j in range( 0, len( conditions ) ):
            if i == j:
                continue
            aData2 = conditions[j]
            
            variance_ratio, P_value = SurvivalTimeFTest( aData1, aData2 )
            Corrected_P_value = min( P_value * ( len( conditions ) - 1 ), 1.0 )

            if aData1.NormalDistribution == True and aData2.NormalDistribution == True:
                warning_message = ""
            else:
                warning_message = "not applicable"
                
            results_var1Test.append( [ aData1.name, aData2.name, variance_ratio, P_value, Corrected_P_value, warning_message ] )

    return results_var1Test

def RunNormalityTestAll( conditions, cutoff = 0.01 ):
    results_normalityTest = []
    for i in range( 0, len( conditions ) ):
        aData1 = conditions[i]
        diff_s_list = [ 0 ]
        for j in range( 1, len( aData1.timelist ) ):
            diff_s_list.append( int( (aData1.km_data_list[j-1].s_hat - aData1.km_data_list[j].s_hat) * aData1.total ) )
            
        estimated = GetFreqData( aData1.timelist, diff_s_list )
        result = rpy.r.shapiro_test( estimated )
        results_normalityTest.append( [ aData1.name, len(estimated), result['statistic']['W'], result['p.value'] ] )

        if result['p.value'] < cutoff:
            aData1.NormalDistribution = False
        else:
            aData1.NormalDistribution = True
        
    return results_normalityTest


if __name__ == "__main__":    
    conditions = SurvivalData.ReadDataFromFile( "sample.txt" )
    SurvivalData.CalculateSubplotData( conditions )

    results_normalityTest = RunNormalityTestAll( conditions )
    results_var1Test = RunSurvivalTimeFTestAll( conditions )

    print "Sapiro-Wilk Normality Test"
    print "\t".join( [ "Sample Name", "Number of individual", "W-value", "P-value" ] )
    for [name, n, wvalue, pvalue] in results_normalityTest:
        print "%s\t%d\t%f\t%f" % ( name, n, wvalue, pvalue )

    print "\nSurvival Time F-Test"
    print "\t".join( [ "Sample 1", "Sample 2", "Ratio of variances", "P-value", "Corrected P-value", "Warning message" ] )
    for results in results_var1Test:
        for v in results:
            print v, "\t",
        print ""




